#!/usr/bin/env python

"""
Handlers for a person for whom scheduling is performed, inspecting outgoing
messages to obtain scheduling done externally.
"""

from imiptools.content import Handler
from vCalendar import to_node

class PersonHandler(Handler):

    "Handling mechanisms specific to people."

    def _record(self, objtype, from_organiser=True, update_freebusy=False):
        identity = self.get_value(from_organiser and "ORGANIZER" or "ATTENDEE")

        # Check for event using UID.

        if not self.have_new_object(identity, objtype):
            return False

        # Store the object.

        self.store.set_event(identity, self.uid, to_node(
            {objtype : [(self.details, {})]}
            ))

        # Update free/busy information.

        if update_freebusy:

            # If newer than any old version, discard old details from the
            # free/busy record and check for suitability.

            periods = self.get_periods()
            freebusy = self.store.get_freebusy(identity) or []

            self.update_freebusy(freebusy, identity, periods)

            if self.publisher:
                self.publisher.set_freebusy(identity, freebusy)

        return True

class Event(PersonHandler):

    "An event handler."

    def add(self):
        pass

    def cancel(self):
        pass

    def counter(self):
        pass

    def declinecounter(self):
        pass

    def publish(self):
        self._record("VEVENT", True, True)

    def refresh(self):
        self._record("VEVENT", True, True)

    def reply(self):
        self._record("VEVENT", False, True)

    def request(self):
        self._record("VEVENT", True, True)

class Freebusy(PersonHandler):

    "A free/busy handler."

    def publish(self):
        pass

    def reply(self):
        pass

    def request(self):
        pass

class Journal(PersonHandler):

    "A journal entry handler."

    def add(self):
        pass

    def cancel(self):
        pass

    def publish(self):
        self._record("VJOURNAL", True)

class Todo(PersonHandler):

    "A to-do item handler."

    def add(self):
        pass

    def cancel(self):
        pass

    def counter(self):
        pass

    def declinecounter(self):
        pass

    def publish(self):
        self._record("VTODO", True)

    def refresh(self):
        self._record("VTODO", True)

    def reply(self):
        self._record("VTODO", False)

    def request(self):
        self._record("VTODO", True)

# Handler registry.

handlers = [
    ("VFREEBUSY",   Freebusy),
    ("VEVENT",      Event),
    ("VTODO",       Todo),
    ("VJOURNAL",    Journal),
    ]

# vim: tabstop=4 expandtab shiftwidth=4
